#!/usr/bin/python
#
# This script downloads regulary the data from the TFD128 and compares
# the collected environment value with some thresholds. If required,
# this function executes some action, e.g. sending an e-mail.
#
# It is meant to be called via cronjob regulary, e.g.:
# # m h  dom mon dow   command
# */10 * * * *    /usr/bin/python envcontrol.py >> tfd128.log
#
# (C) 2007 by Pascal Bihler <bihler@iai.uni-bonn.de>
#
# TODO: * Internationalisation
#       * Better error handling
#       * Add plot created with gnuplot as file attachment to the sent e-mail
#       * Send warning if device is not available
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from __future__ import print_function

# We use the Tfd128-module from Andreas Engel
from tfd128 import Tfd128
logger = Tfd128('/dev/tfd128')

# This data may be adjusted:
MODE        = logger.TEMPERATURE | logger.HUMIDITY
INTERVAL    = 1
TEMP_LIMITS = ( 10, 40 )
HUM_LIMITS  = ( 10, 70 ) # if set logger.HUMIDITY
WARNING     = "Warning: Environmental limits exceeded!"
RECEIVER    = "me@example.com"
SENDER      = "tfd128@example.com"
MAIL        = "/usr/sbin/sendmail"

def limit_action(extreme_temp, extreme_hum, data, humset = False):
    import os
    warning = WARNING + "\n\n"
    if extreme_temp[0] < TEMP_LIMITS[0]:
        warning += "Temperature was too low: %s\n" % extreme_temp[0]
    if extreme_temp[1] > TEMP_LIMITS[1]:
        warning += "Temperature was too high: %s\n" % extreme_temp[1]
    if humset and (extreme_hum[0] < HUM_LIMITS[0]):
        warning += "Humidity was too low: %s\n" % extreme_hum[0]
    if humset and (extreme_hum[1] > HUM_LIMITS[1]):
        warning += "Humidity was too high: %s\n" % extreme_hum[1]

    warning += "\n---- Data ----\n" + data

    # open a pipe to the mail program and
    # write the data to the pipe
    p = os.popen("%s -t" % MAIL, 'w')
    p.write("To: "      + RECEIVER + "\n"
            "From: "    + SENDER   + "\n"
            "Subject: " + WARNING  + "\n"
            "\n" + warning)
    p.close()

# -------------------------------------------

extreme_temp = (1000, -1000) # something we never get
extreme_hum  = (1000, -1000) # something we never get

# Stop the logger, if still running:
if logger.is_busy():
    logger.stop()

data = ""
try:
    # Now get the data from the logger
    logger.time_format = "%Y-%m-%dh%H:%M:%S"
    params = logger.params()
    hasHumidity = params['mode'] & logger.HUMIDITY
    for v in logger.data():
        extreme_temp[0] = min(extreme_temp[0], v[1])
        extreme_temp[1] = max(extreme_temp[1], v[1])
        if hasHumidity:
            extreme_hum[0] = min(extreme_hum[0], v[2])
            extreme_hum[1] = max(extreme_hum[1], v[2])
            v = "%s;%4.1f;%d" % v
        else:
            v = "%s;%4.1f" % v
        # Print the values to stdout (for logging reasons, if you want
        # to log, to create nice images etc...)
        print(v)
        data += "%s\n" % v

finally:
    # Check limit and call action, if required
    if (extreme_temp[0] < TEMP_LIMITS[0]) \
            or (extreme_temp[1] > TEMP_LIMITS[1]) \
            or (hasHumidity and
                (extreme_hum[0] < HUM_LIMITS[0]) \
                    or (extreme_hum[1] > HUM_LIMITS[1])):
        limit_action(extreme_temp, extreme_hum, data, hasHumidity)

    # Now (re)start the logger:
    logger.start(INTERVAL, MODE)
